﻿using System.Linq;
using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Encodings.BinaryVectorEncoding;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

namespace HeuristicLab.Problems.NK {

  [Item("Average Bit Binary Vector Comparer", "Compares two binary vectors by their average positive bit location")]
  [StorableClass]
  public sealed class AverageBitBinaryVectorComparer : BinaryVectorComparer {

    [StorableConstructor]
    private AverageBitBinaryVectorComparer(bool deserializing) : base(deserializing) { }
    private AverageBitBinaryVectorComparer(AverageBitBinaryVectorComparer original, Cloner cloner)
      : base(original, cloner) {
    }
    public AverageBitBinaryVectorComparer() { }
    public override IDeepCloneable Clone(Cloner cloner) {
      return new AverageBitBinaryVectorComparer(this, cloner);
    }

    public static double AverageBit(BinaryVector x) {
      return x.Select((b, i) => new { b, i }).Where(v => v.b).Average(v => v.i);
    }

    public override int Compare(BinaryVector x, BinaryVector y) {
      return (AverageBit(x) - AverageBit(y)).CompareTo(0);
    }
  }
}
